@@ -33,7 +33,10 @@ import attr
 
 from hypothesis.errors import InvalidArgument, HypothesisDeprecationWarning
 from hypothesis.configuration import hypothesis_home_dir
+from hypothesis.internal.compat import text_type
 from hypothesis.utils.conventions import UniqueIdentifier, not_set
+from hypothesis.internal.reflection import proxies, \
+    get_pretty_function_description
 from hypothesis.internal.validation import try_convert
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
@@ -174,8 +177,63 @@ class settings(settingsMeta('settings', (object,), {})):
             return self.storage.defaults_stack
 
     def __call__(self, test):
+        """Make the settings object (self) an attribute of the test.
+
+        The settings are later discovered by looking them up on the test
+        itself.
+
+        Also, we want to issue a deprecation warning for settings used alone
+        (without @given) so, note the deprecation in the new test, but also
+        attach the version without the warning as an attribute, so that @given
+        can unwrap it (since if @given is used, that means we don't want the
+        deprecation warning).
+
+        When it's time to turn the warning into an error, we'll raise an
+        exception instead of calling note_deprecation (and can delete
+        "test(*args, **kwargs)").
+        """
+        if not callable(test):
+            raise InvalidArgument(
+                'settings objects can be called as a decorator with @given, '
+                'but test=%r' % (test,)
+            )
+        if hasattr(test, '_hypothesis_internal_settings_applied'):
+            note_deprecation(
+                '%s has already been decorated with a settings object, which '
+                'will be overridden.  This will be an error in a future '
+                'version of Hypothesis.\n    Previous:  %r\n    This:  %r' % (
+                    get_pretty_function_description(test),
+                    test._hypothesis_internal_use_settings,
+                    self
+                )
+            )
+
         test._hypothesis_internal_use_settings = self
-        return test
+
+        # For double-@settings check:
+        test._hypothesis_internal_settings_applied = True
+
+        @proxies(test)
+        def new_test(*args, **kwargs):
+            note_deprecation(
+                'Using `@settings` without `@given` does not make sense and '
+                'will be an error in a future version of Hypothesis.'
+            )
+            test(*args, **kwargs)
+
+        # @given will get the test from this attribution (rather than use the
+        # version with the deprecation warning)
+        new_test._hypothesis_internal_test_function_without_warning = test
+
+        # This means @given has been applied, so we don't need to worry about
+        # warning for @settings alone.
+        has_given_applied = getattr(test, 'is_hypothesis_test', False)
+        test_to_use = test if has_given_applied else new_test
+        test_to_use._hypothesis_internal_use_settings = self
+        # Can't use _hypothesis_internal_use_settings as an indicator that
+        # @settings was applied, because @given also assigns that attribute.
+        test._hypothesis_internal_settings_applied = True
+        return test_to_use
 
     @classmethod
     def define_setting(
@@ -284,41 +342,53 @@ class settings(settingsMeta('settings', (object,), {})):
         return default_context_manager.__exit__(*args, **kwargs)
 
     @staticmethod
-    def register_profile(name, settings):
+    def register_profile(name, parent=None, **kwargs):
         """Registers a collection of values to be used as a settings profile.
 
-        These settings can be loaded in by name. Enable different
-        defaults for different settings.  ``settings`` must be a
-        settings object.
+        Settings profiles can be loaded by name - for example, you might
+        create a 'fast' profile which runs fewer examples, keep the 'default'
+        profile, and create a 'ci' profile that increases the number of
+        examples and uses a different database to store failures.
+
+        The arguments to this method are exactly as for
+        :class:`~hypothesis.settings`: optional ``parent`` settings, and
+        keyword arguments for each setting that will be set differently to
+        parent (or settings.default, if parent is None).
         """
-        settings._profiles[name] = settings
+        if not isinstance(name, (str, text_type)):
+            note_deprecation('name=%r must be a string' % (name,))
+        if 'settings' in kwargs:
+            if parent is None:
+                parent = kwargs.pop('settings')
+                note_deprecation('The `settings` argument is deprecated - '
+                                 'use `parent` instead.')
+            else:
+                raise InvalidArgument(
+                    'The `settings` argument is deprecated, and has been '
+                    'replaced by the `parent` argument.  Use `parent` only.'
+                )
+        settings._profiles[name] = settings(parent=parent, **kwargs)
 
     @staticmethod
     def get_profile(name):
-        """Return the profile with the given name.
-
-        - name is a string representing the name of the profile
-         to load
-        A InvalidArgument exception will be thrown if the
-         profile does not exist
-        """
+        """Return the profile with the given name."""
+        if not isinstance(name, (str, text_type)):
+            note_deprecation('name=%r must be a string' % (name,))
         try:
             return settings._profiles[name]
         except KeyError:
-            raise InvalidArgument(
-                "Profile '{0}' has not been registered".format(
-                    name
-                )
-            )
+            raise InvalidArgument('Profile %r is not registered' % (name,))
 
     @staticmethod
     def load_profile(name):
-        """Loads in the settings defined in the profile provided If the profile
-        does not exist an InvalidArgument will be thrown.
+        """Loads in the settings defined in the profile provided.
 
+        If the profile does not exist, InvalidArgument will be raised.
         Any setting not defined in the profile will be the library
-        defined default for that setting
+        defined default for that setting.
         """
+        if not isinstance(name, (str, text_type)):
+            note_deprecation('name=%r must be a string' % (name,))
         settings._current_profile = name
         settings._assign_default_internal(settings.get_profile(name))
 
@@ -339,8 +409,12 @@ settings.define_setting(
     default=5,
     description="""
 Raise Unsatisfiable for any tests which do not produce at least this many
-values that pass all assume() calls and which have not exhaustively covered the
-search space.
+values that pass all :func:`hypothesis.assume` calls and which have not
+exhaustively covered the search space.
+
+Note that examples are compared at the level of the underlying byte-stream -
+for example, :func:`~hypothesis.strategies.booleans` contains 256 unique
+examples at this level because it always uses one byte.
 """
 )
 
@@ -509,6 +583,9 @@ class HealthCheck(Enum):
     large_base_example = 7
     """Checks if the natural example to shrink towards is very large."""
 
+    not_a_test_method = 8
+    """Checks if @given has been applied to a method of unittest.TestCase."""
+
 
 @unique
 class Statistics(IntEnum):
@@ -716,10 +793,6 @@ more details of this behaviour.
 
 settings.lock_further_definitions()
 
-settings.register_profile('default', settings())
-settings.load_profile('default')
-assert settings.default is not None
-
 
 def note_deprecation(message, s=None):
     if s is None:
@@ -729,3 +802,8 @@ def note_deprecation(message, s=None):
     warning = HypothesisDeprecationWarning(message)
     if verbosity > Verbosity.quiet:
         warnings.warn(warning, stacklevel=3)
+
+
+settings.register_profile('default', settings())
+settings.load_profile('default')
+assert settings.default is not None
